@@ -29,15 +29,22 @@ would be appreciated.
 
 """
 
+from __future__ import (absolute_import, division, print_function)
+
 import warnings
-import urlparse
-import urllib2
-from urllib import urlencode
-from etree import etree
+import six
+from six.moves import filter
+try:                    # Python 3
+    from urllib.parse import (urlencode, urlparse, urlunparse, parse_qs,
+                              ParseResult)
+except ImportError:      # Python 2
+    from urllib import urlencode
+    from urlparse import urlparse, urlunparse, parse_qs, ParseResult
+from .etree import etree
 from .util import openURL, testXMLValue, getXMLInteger
-from fgdc import Metadata
-from iso import MD_Metadata
-from ows import ServiceProvider, ServiceIdentification, OperationsMetadata
+from .fgdc import Metadata
+from .iso import MD_Metadata
+from .ows import ServiceProvider, ServiceIdentification, OperationsMetadata
 
 
 _OWS_NS = '{http://www.opengis.net/ows/1.1}'
@@ -113,7 +120,7 @@ class WebMapTileService(object):
     def __getitem__(self, name):
         '''Check contents dictionary to allow dict like access to
         service layers'''
-        if name in self.__getattribute__('contents').keys():
+        if name in self.__getattribute__('contents'):
             return self.__getattribute__('contents')[name]
         else:
             raise KeyError("No content named %s" % name)
@@ -291,7 +298,7 @@ TILEMATRIX=6&TILEROW=4&TILECOL=4&FORMAT=image%2Fjpeg'
         if (layer is None):
             raise ValueError("layer is mandatory (cannot be None)")
         if style is None:
-            style = self[layer].styles.keys()[0]
+            style = list(self[layer].styles.keys())[0]
         if format is None:
             format = self[layer].formats[0]
         if tilematrixset is None:
@@ -316,7 +323,7 @@ TILEMATRIX=6&TILEROW=4&TILECOL=4&FORMAT=image%2Fjpeg'
         request.append(('TILECOL', str(column)))
         request.append(('FORMAT', format))
 
-        for key, value in kwargs.iteritems():
+        for key, value in six.iteritems(kwargs):
             request.append((key, value))
 
         data = urlencode(request, True)
@@ -366,7 +373,7 @@ TILEMATRIX=6&TILEROW=4&TILECOL=4&FORMAT=image%2Fjpeg'
                                    tilematrix='6',\
                                    row=4, column=4)
             >>> out = open('tile.jpg', 'wb')
-            >>> out.write(img.read())
+            >>> bytes_written = out.write(img.read())
             >>> out.close()
 
         """
@@ -378,9 +385,9 @@ TILEMATRIX=6&TILEROW=4&TILECOL=4&FORMAT=image%2Fjpeg'
         if base_url is None:
             base_url = self.url
             try:
-                get_verbs = filter(
-                    lambda x: x.get('type').lower() == 'get',
-                    self.getOperationByName('GetTile').methods)
+                methods = self.getOperationByName('GetTile').methods
+                get_verbs = [x for x in methods
+                             if x.get('type').lower() == 'get']
                 if len(get_verbs) > 1:
                     # Filter by constraints
                     base_url = next(
@@ -388,8 +395,7 @@ TILEMATRIX=6&TILEROW=4&TILECOL=4&FORMAT=image%2Fjpeg'
                             list,
                             ([pv.get('url')
                                 for const in pv.get('constraints')
-                                if 'kvp' in map(
-                                    lambda x: x.lower(), const.values)]
+                                if 'kvp' in [x.lower() for x in const.values]]
                              for pv in get_verbs if pv.get('constraints'))))[0]
                 elif len(get_verbs) == 1:
                     base_url = get_verbs[0].get('url')
@@ -402,7 +408,7 @@ TILEMATRIX=6&TILEROW=4&TILECOL=4&FORMAT=image%2Fjpeg'
         if u.info()['Content-Type'] == 'application/vnd.ogc.se_xml':
             se_xml = u.read()
             se_tree = etree.fromstring(se_xml)
-            err_message = unicode(se_tree.find('ServiceException').text)
+            err_message = six.text_type(se_tree.find('ServiceException').text)
             raise ServiceException(err_message.strip(), se_xml)
         return u
 
@@ -701,8 +707,8 @@ class WMTSCapabilitiesReader:
         """
         # Ensure the 'service', 'request', and 'version' parameters,
         # and any vendor-specific parameters are included in the URL.
-        pieces = urlparse.urlparse(service_url)
-        args = urlparse.parse_qs(pieces.query)
+        pieces = urlparse(service_url)
+        args = parse_qs(pieces.query)
         if 'service' not in args:
             args['service'] = 'WMTS'
         if 'request' not in args:
@@ -712,10 +718,10 @@ class WMTSCapabilitiesReader:
         if vendor_kwargs:
             args.update(vendor_kwargs)
         query = urlencode(args, doseq=True)
-        pieces = urlparse.ParseResult(pieces.scheme, pieces.netloc,
-                                      pieces.path, pieces.params,
-                                      query, pieces.fragment)
-        return urlparse.urlunparse(pieces)
+        pieces = ParseResult(pieces.scheme, pieces.netloc,
+                             pieces.path, pieces.params,
+                             query, pieces.fragment)
+        return urlunparse(pieces)
 
     def read(self, service_url, vendor_kwargs=None):
         """Get and parse a WMTS capabilities document, returning an
@@ -738,7 +744,7 @@ class WMTSCapabilitiesReader:
 
         string should be an XML capabilities document
         """
-        if not isinstance(st, str):
-            msg = 'String must be of type string, not %s' % type(st)
+        if not isinstance(st, str) and not isinstance(st, bytes):
+            msg = 'String must be of type string or bytes, not %s' % type(st)
             raise ValueError(msg)
         return etree.fromstring(st)
